-- **************************************************************************
-- Owner:		Xilinx Inc.
-- File:  		dec_func.vhd
--
-- Purpose: 		Main 8B/10B decoder function.  Decodes serial data
--			stream following 5B/6B and 3B/4B encoding schemes.  Decodes
--			incoming 10-bit transmitted serial data stream of ain .. jin
--			to byte wide data AOUT .. HOUT.  Incoming data includes control
--			characters i and j.  Detects transmission of special characters 
--			and asserts KOUT respectively.
--
-- Author:		Jennifer Jenkins
-- Date:		3-31-2000
--
-- **************************************************************************


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity DEC_FUNC is
    port(
        
     clk			: in STD_LOGIC;
          -- Data Inputs (ain .. jin)
	  b10in		: in STD_LOGIC_vector(9 downto 0);			
	  -- Output data terms (AOUT .. HOUT)
	  b8out		: out STD_LOGIC_vector(7 downto 0);

	  kout		: out STD_LOGIC		-- Asserted when special character
								-- is detected in transmission
          );

end DEC_FUNC;



architecture BEHAVIOUR of DEC_FUNC is
signal ain,bin,cin,din,ein,fin,gin,hin,iin,jin: std_logic;
-- Preliminary output signals
signal a_cl1, a_cl2, a_cl3 : STD_LOGIC;
signal b_cl1, b_cl2, b_cl3 : STD_LOGIC;
signal c_cl1, c_cl2, c_cl3 : STD_LOGIC;
signal d_cl1, d_cl2, d_cl3 : STD_LOGIC;
signal e_cl1, e_cl2, e_cl3 : STD_LOGIC;
signal f_sel1, f_sel2 : STD_LOGIC;
signal g_cl1, g_cl2 : STD_LOGIC;
signal h_cl1, h_cl2 : STD_LOGIC;
signal k_cl1, k_cl2 : STD_LOGIC;
begin
ain <= b10in(0);
bin <= b10in(1);
cin <= b10in(2);
din <= b10in(3);
ein <= b10in(4);
iin <= b10in(5);
fin <= b10in(6);
gin <= b10in(7);
hin <= b10in(8);
jin <= b10in(9);
	-- **************** SIGNAL ASSIGNMENTS *************************
	a_cl1 <= ((ein or not(iin) or (not(cin xor bin) or din)) and
		 (ein or not(bin and din and
		 (not(iin or cin) or (not(ain) and iin and cin)))));

	a_cl2 <= ((not(ein) or ((din or bin) and (iin or cin))) and
		 (iin or (not(cin) or (din and bin))));

	a_cl3 <= (not(cin) and din and iin and not(((ain and ein) or (bin and not(ein)))));

	b_cl1 <= ((not(bin) and (din xor ein) and iin) or not(iin or din or ein)); 

	b_cl2 <= (bin and not(iin) and 
		 not(((ain and cin) or not(din or ein)) and (not(ein) or din)));

	b_cl3 <= (not((not((bin and ein) or (bin and din) or (iin and din)) or ain or cin) and
		    (din or ein or not(iin) or not(ain xor cin))));

	c_cl1 <= (not(ain) and din and iin and (not(bin) or (not(cin) and ein)));

	c_cl2 <= (not((ain and bin and not(cin) and din and iin) or
		 (not(iin or not(bin) or not(ain xor din))) or
		 (not(din) and ((not(ain) and bin and iin) or (ain and not(bin) and iin)))));

	c_cl3 <= (cin and not(((not(ain) and not(ein)) or (ain and bin) or iin) and
			     ((iin and bin) or din or not(ein))));

	d_cl1 <= ((not(ain) and (din or (iin and not(ein))) and (cin xor bin)) or
		 (ain and (cin xor bin) and ((not(din) and ein and iin) or
					     (not(ein) and not(iin)))));

	d_cl2 <= (not(ein or not(iin) or (not(ain and not(bin) and not(cin)) and
					 (not(ain) or not(bin) or not(cin) or din))));
 
	d_cl3 <= ((not(bin) and ein) or (bin and not(ain and cin)));
				
	e_cl1 <= ((not(iin) and bin and ((ain and not(cin) and not(din)) or 
				(ein and (ain xor cin)))) or
		 (not(bin or not(din) or ((not(ain) or not(ein) or iin) and
					  ((ain or cin or ein or not(iin)))))));
					       
	e_cl2 <= ((cin and not((ain or (bin and din) or (not(bin) and not(iin))) and
			      ((iin and bin) or not(ain) or din))) or
		 not(cin or din or not(bin) or not(iin)));

	e_cl3 <= ((cin or ain or not(bin)) and (bin or not(ain xor cin)));

	f_sel1 <= (not(hin and not(iin or ein or din or cin)));

	f_sel2 <= (hin or not(iin or ein or din or cin));
				
	g_cl1 <= (((not(not(fin) or (hin and (iin or ein or din or cin)))) or
		  fin or (hin and (iin or ein or din or cin))) and
		 ((not(not(fin) or (hin and (iin or ein or din or cin)))) or
		  gin));

	g_cl2 <= (not((fin or not(iin or ein or hin or din or cin)) and
		 ((gin or hin or iin or ein or din or cin) and (gin or fin))));

	h_cl1 <= (not((jin and not(iin or ein or din or cin)) or 
		     (fin and not(jin or gin))));

	h_cl2 <= ((hin and not(jin) and (iin or ein or din or cin)) or 
		 ((hin xor jin) and not(gin xor fin)));

	k_cl1 <= (not(((ain xor bin) and cin and din) or 
		     (ain and bin and not(iin or not(din xor cin)))));

	k_cl2 <= ((not(ain or bin) and (iin and (din xor cin))) or 
		 ((ain xor bin) and not(cin or din)));
	-- ***************** Process: SEQUENTIAL ************************
    	-- Purpose: 	Decoding control logic.  Synchronize control logic 
	-- 		for decoding data inputs.
    
    	SEQUENTIAL: process (clk)
    	begin
        if clk'event and (clk = '1') then
				-- Generate the output data 
				b8out(0) <= (not(a_cl2) and ain) or
					not(a_cl1) or a_cl3;
				b8out(1) <= (b_cl1 and ain and cin) or
					b_cl2 or b_cl3;
				b8out(2) <= not((ein or c_cl2) and not(c_cl1) and not(c_cl3));
				b8out(3) <= (d_cl3 and din and not(iin)) or
					d_cl1 or d_cl2;
				b8out(4) <= (ein and e_cl2) or (not(e_cl3) and (din xor iin)) or e_cl1;
				
				if jin = '0' then
					if f_sel1 = '0' then
						b8out(5) <= gin;
					else 	b8out(5) <= fin;
					end if;
			
					b8out(6) <= g_cl1;

				else	-- jin = '1'
					if f_sel2 = '0' then
						b8out(5) <= not(gin);
					else 	b8out(5) <= not(fin);
					end if;

					b8out(6) <= g_cl2;

				end if;				
				
				b8out(7) <= not((h_cl1 or hin) and not(h_cl2) and
					    not(not(fin) and gin and hin and jin));

				if ein = '0' then
					kout <= (gin and hin and jin and k_cl2) or (not(cin or din or iin));
				else   	-- ein = '1'
					kout <= not(((jin or hin) or k_cl1 or gin) and 
						    (not(cin and din and iin)));
				end if;        
        end if;
    	end process SEQUENTIAL;
end BEHAVIOUR;
































